/*	$Id$	*/

/*
 * Copyright (c) 2012, Internet Initiative Japan, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include <sys/types.h>
#include <inttypes.h>

#include <libarms.h>
#include <libarms_resource.h>
#include <axp_extern.h>
#include <module_db_mi.h>
#include <transaction/transaction.h>

static int curstate = ARMS_ST_INITIAL;

void
arms_set_global_state(int newstate)
{
	arms_context_t *res = arms_get_context();
	int store = 0;

	if (newstate == curstate)
		return;

	if (res != NULL && res->callbacks.state_cb != NULL) {
		res->callbacks.state_cb(
			curstate,
			newstate,
			res->udata);
	}
	curstate = newstate;

	if (newstate == ARMS_ST_PUSH_WAIT) {
		switch (res->mc_state) {
		case MODULE_CACHE_PRE_UPDATE:
		case MODULE_CACHE_PRE_STORE:
			store = 1;
			/* fall through */
		case MODULE_CACHE_PRE_REMOVE:
			update_running_module_info();
			if (store)
				store_dump_module(res);
			else
				remove_dump_module(res);
			purge_all_config();
			arms_module_cache_event(res, MODULE_CACHE_EV_SYNCED);
			break;
		}
	}
}

int
arms_get_global_state(void)
{
	return curstate;
}
